"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const debug_1 = require("debug");
const ScreencastFrameCollector_1 = require("./ScreencastFrameCollector");
const VideoFrameBuilder_1 = require("./VideoFrameBuilder");
const VideoWriter_1 = require("./VideoWriter");
const debug = debug_1.default('playwright-video:PageVideoCapture');
class PageVideoCapture {
    constructor({ collector, page, writer }) {
        this._frameBuilder = new VideoFrameBuilder_1.VideoFrameBuilder();
        this._stopped = false;
        this._collector = collector;
        this._writer = writer;
        this._writer.on('ffmpegerror', error => {
            debug(`stop due to ffmpeg error "${error.trim()}"`);
            this.stop();
        });
        page.on('close', () => this.stop());
        this._listenForFrames();
    }
    static start({ page, savePath, }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            debug('start');
            const collector = yield ScreencastFrameCollector_1.ScreencastFrameCollector.create(page);
            const writer = yield VideoWriter_1.VideoWriter.create(savePath);
            const capture = new PageVideoCapture({ collector, page, writer });
            yield collector.start();
            return capture;
        });
    }
    _listenForFrames() {
        this._collector.on('screencastframe', screencastFrame => {
            debug(`received frame: ${screencastFrame.timestamp}`);
            const videoFrames = this._frameBuilder.buildVideoFrames(screencastFrame);
            this._writer.write(videoFrames);
        });
    }
    _writeLastFrame() {
        debug('write last frame');
        const videoFrames = this._frameBuilder.buildVideoFrames();
        this._writer.write(videoFrames);
    }
    stop() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this._stopped)
                return;
            debug('stop');
            this._stopped = true;
            yield this._collector.stop();
            this._writeLastFrame();
            return this._writer.stop();
        });
    }
}
exports.PageVideoCapture = PageVideoCapture;
//# sourceMappingURL=PageVideoCapture.js.map