"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = require("debug");
const debug = debug_1.default('playwright-video:VideoFrameBuilder');
class VideoFrameBuilder {
    constructor() {
        this._framesPerSecond = 25;
    }
    _getFrameCount(screencastFrame) {
        let durationSeconds;
        if (screencastFrame) {
            durationSeconds =
                screencastFrame.timestamp - this._previousFrame.timestamp;
        }
        else {
            durationSeconds = (Date.now() - this._previousFrame.received) / 1000;
        }
        return Math.round(durationSeconds * this._framesPerSecond);
    }
    buildVideoFrames(screencastFrame) {
        if (!this._previousFrame) {
            debug('first frame received: waiting for more');
            this._previousFrame = screencastFrame;
            return [];
        }
        const frameCount = this._getFrameCount(screencastFrame);
        if (frameCount < 0) {
            debug('frames out of order: skipping frame');
            return [];
        }
        const frames = Array(frameCount).fill(this._previousFrame.data);
        debug(`returning ${frames.length} frames`);
        this._previousFrame = screencastFrame;
        return frames;
    }
}
exports.VideoFrameBuilder = VideoFrameBuilder;
//# sourceMappingURL=VideoFrameBuilder.js.map