"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const debug_1 = require("debug");
const events_1 = require("events");
const ffmpeg = require("fluent-ffmpeg");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const stream_1 = require("stream");
const utils_1 = require("./utils");
const debug = debug_1.default('playwright-video:VideoWriter');
class VideoWriter extends events_1.EventEmitter {
    constructor(savePath) {
        super();
        this._framesPerSecond = 25;
        this._receivedFrame = false;
        this._stopped = false;
        this._stream = new stream_1.PassThrough();
        utils_1.ensureFfmpegPath();
        this._writeVideo(savePath);
    }
    static create(savePath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield fs_extra_1.ensureDir(path_1.dirname(savePath));
            return new VideoWriter(savePath);
        });
    }
    _writeVideo(savePath) {
        debug(`write video to ${savePath}`);
        this._endedPromise = new Promise((resolve, reject) => {
            ffmpeg({ source: this._stream, priority: 20 })
                .videoCodec('libx264')
                .inputFormat('image2pipe')
                .inputFPS(this._framesPerSecond)
                .outputOptions('-preset ultrafast')
                .on('error', e => {
                this.emit('ffmpegerror', e.message);
                if (!this._receivedFrame &&
                    e.message.includes('pipe:0: End of file')) {
                    resolve();
                    return;
                }
                reject(`playwright-video: error capturing video: ${e.message}`);
            })
                .on('end', () => {
                resolve();
            })
                .save(savePath);
        });
    }
    stop() {
        if (this._stopped) {
            return this._endedPromise;
        }
        this._stopped = true;
        this._stream.end();
        return this._endedPromise;
    }
    write(frames) {
        debug(`write ${frames.length} frames`);
        if (frames.length && !this._receivedFrame) {
            this._receivedFrame = true;
        }
        frames.forEach(frame => {
            this._stream.write(frame);
        });
    }
}
exports.VideoWriter = VideoWriter;
//# sourceMappingURL=VideoWriter.js.map